/*
 * Provide the basic functions to control the peripherials on
 * the HTL's Gemini Board using the ATmega32 daughterboard
 *
 * Autor: Bernhard Wrndl-Aichriedler 
 * E-Mail: bwa@berniwa.com 
 * Homepage: http://www.berniwa.com
 */

/**
 * @defgroup blib_gemini Gemini Library
 *
 * @brief Gemini Board peripherial control library
 *
 * Provide the basic functions to control the peripherials on
 * on the HTL's Gemini board, with the ATmega32 daughterboard
 *
 * @author Bernhard Wrndl-Aichriedler bwa@berniwa.com http://www.berniwa.com
 */


#ifndef _GEMINI_H

	#include <avr/io.h>
	#include "config.h"
	#include "default.h"
	#include "i2c.h"
	#include "rs232.h"
	#include "lcd.h"
	#include "spi.h"
	#include "util/delay.h"
	#include "gemini.h"


	/** 
	 * @brief 	Intializes all peripherials on the Gemini Board
	 * @param 	void
	 * @return 	none
	 */
	void geminiInit()
	{
		i2cInit();
		
		lcdInit();
		lcdClear();	
		rs232Init();

		spiMasterInit();
	}

	/** 
	 * @brief 	Write a value to the 16bit shift register leds
	 * @param 	value Value to write
	 * @return 	none
	 */
	void geminiLedWrite(unsigned int value)
	{
		spiMasterWrite(value >> 8);
		spiMasterWrite(value & 255);
		DDRB |= (1 << PB6);
		PORTB |= (1 << PB6);
		_delay_us(5);
		PORTB &= ~(1 << PB6);	
	}

	/** 
	 * @brief 	Write a value to one of the digital potentiometers
	 * @param 	channel	Pot. channel (0 or 1)
	 * @param 	value 	Value to write
	 * @return 	none
	 */
	void geminiPotWrite(unsigned char channel, unsigned char value)
	{
		DDRB |= (1 << PB4);
		PORTB &= ~(1 << PB4);
		spiMasterWrite(B8(00010000) | (channel+1));
		spiMasterWrite(value);
		PORTB |= (1 << PB4);	
	}

	/** 
	 * @brief 	Read the value of the io extender
	 * @param 	none
	 * @return	Port value
	 */
	unsigned char geminiIoRead()
	{
		return i2cDirectRead(GEMINI_I2C_IO);
	}

	/** 
	 * @brief 	Write a value to the io extender
	 * @param 	value 	Value to write
	 * @return 	none
	 */
	void geminiIoWrite(unsigned char value)
	{
		i2cDirectWrite(GEMINI_I2C_IO, value);
	}

	/** 
	 * @brief 	Read the time from the real time clock
	 * @param 	type	Field to be read (TIME_HS for 1/100s, TIME_S for 1s, TIME_M for 1m, TIME, H for 1h) 	
	 * @return 	Time value
	 */
	unsigned char geminiTimeRead(unsigned char type)
	{
		unsigned char tmp = 0;
		tmp = i2cRegisterRead(GEMINI_I2C_TIME, type);
		return (tmp & B8(00001111)) + (tmp >> 4) * 10;
	}

	/** 
	 * @brief 	Set the time of the real time clock
	 * @param 	type 	Field to be set (TIME_HS for 1/100s, TIME_S for 1s, TIME_M for 1m, TIME, H for 1h) 		
	 * @param 	value Value to write
	 * @return 	none
	 */
	void geminiTimeWrite(unsigned char type, unsigned char value)
	{
		i2cRegisterWrite(GEMINI_I2C_TIME, type, ((value / 10) << 4) + (value%10));
	}

	/** 
	 * @brief 	Read the current voltage from one of the channels of the ad converter
	 * @param 	channel	Channel to be read (0 to 3)
	 * @return 	AD value
	 */
	unsigned char geminiAdRead(unsigned char channel)
	{
		return i2cRegisterRead(GEMINI_I2C_AD, B8(01000000) | channel);
	}

	/** 
	 * @brief 	Set output voltage of DA converter
	 * @param 	value	Value for DA conversion	
	 * @return 	none
	 */
	void geminiDaWrite(unsigned char value)
	{
	
		i2cRegisterWrite(GEMINI_I2C_AD, B8(01000000), value);
	}



	/** 
	 * @brief 	Set Data Direction Register value of Port A (connected over 16bit io extender)
	 * @param 	value	DDR value	
	 * @return 	none
	 */
	void geminiDDRAWrite(unsigned char value)
	{
		/*TODO*/
	}

	/** 
	 * @brief 	Read Data Direction Register value of Port A (connected over 16bit io extender)
	 * @param	none
	 * @return 	DDR value
	 */
	unsigned char geminiDDRARead()
	{
		/*TODO*/
		return 0;
	}

	/** 
	 * @brief 	Set Port Value of Port A (connected over 16bit io extender)
	 * @param 	value	DDR value	
	 * @return 	none
	 */
	void geminiPORTAWrite(unsigned char value)
	{
		/*TODO*/
	}

	/** 
	 * @brief 	Read Port value of Port A (connected over 16bit io extender)
	 * @param	none
	 * @return 	Port value
	 */
	unsigned char geminiPORTARead()
	{
		/*TODO*/
		return 0;
	}

	/** 
	 * @brief 	Read Pin value of Port A (connected over 16bit io extender)
	 * @param	none
	 * @return 	Pin value
	 */
	unsigned char geminiPINARead()
	{
		/*TODO*/
		return 0;
	}

	/** 
	 * @brief 	Set Data Direction Register value of Port C (connected over 16bit io extender)
	 * @param 	value	DDR value	
	 * @return 	none
	 */
	void geminiDDRCWrite(unsigned char value)
	{
		/*TODO*/
	}

	/** 
	 * @brief 	Read Data Direction Register value of Port C (connected over 16bit io extender)
	 * @param	none
	 * @return 	DDR value
	 */
	unsigned char geminiDDRCRead()
	{
		/*TODO*/
		return 0;
	}

	/** 
	 * @brief 	Set Port Value of Port C (connected over 16bit io extender)
	 * @param 	value	DDR value	
	 * @return 	none
	 */
	void geminiPORTCWrite(unsigned char value)
	{
		/*TODO*/
	}

	/** 
	 * @brief 	Read Port value of Port C (connected over 16bit io extender)
	 * @param	none
	 * @return 	Port value
	 */
	unsigned char geminiPORTCRead()
	{
		/*TODO*/
		return 0;
	}

	/** 
	 * @brief 	Read Pin value of Port C (connected over 16bit io extender)
	 * @param	none
	 * @return 	Pin value
	 */
	unsigned char geminiPINCRead()
	{
		/*TODO*/
		return 0;
	}

#endif
