/*
 * Provide the basic functions to control the peripherials on
 * the HTL's Gemini board, using the ATmega32 daughterboard
 *
 * Autor: Bernhard Wrndl-Aichriedler 
 * E-Mail: bwa@berniwa.com 
 * Homepage: http://www.berniwa.com
 */

/**
 * @defgroup blib_gemini Gemini Library
 *
 * @brief Gemini Board peripherial control library
 *
 * Provide the basic functions to control the peripherials on
 * on the HTL's Gemini board, with the ATmega32 daughterboard
 *
 * @author Bernhard Wrndl-Aichriedler bwa@berniwa.com http://www.berniwa.com
 */

#ifndef _GEMINI_H
#define _GEMINI_H 1

	#include <avr/io.h>

	/*
	 * Redeclare ports to match gemini naming
	 */
	#define GEMINI_PORTB 	PORTA
	#define GEMINI_DDRB 	DDRA
	#define GEMINI_PINB 	PINA

	#define GEMINI_PORTD 	PORTB
	#define GEMINI_DDRD		DDRB
	#define GEMINI_PIND 	PINB


	/*
	 * Port B signal declaration
	 */
	#define GEMINI_CSDP 	PB4
	#define GEMINI_SRSTR 	PB6
	#define GEMINI_SRCLK	PB7
	#define GEMINI_SRDIN	PB5

	#define GEMINI_AD4		PB3
	#define GEMINI_AD5		PB2
	#define GEMINI_AD6		PB1
	#define GEMINI_AD7		PB0


	/*
	 * Port C signal declaration
	 */
	#define GEMINI_PKBWR	PC7
	#define GEMINI_PKBRD	PC6

	#define GEMINI_TDI		PC5
	#define GEMINI_TDO		PC4
	#define GEMINI_TMS		PC3
	#define GEMINI_TCK		PC2

	#define GEMINI_SDA		PC1
	#define GEMINI_SCL		PC0


	/*
	 * Port D signal declaration
	 */
	#define GEMINI_LCDEN	PD7
	#define GEMINI_LCDRS	PD6

	#define GEMINI_DQ		PD5

	#define GEMINI_DP		PD4
	#define GEMINI_DM		PD2

	#define GEMINI_XINT		PD3

	#define GEMINI_TXD		PD1
	#define GEMINI_RXD		PD0

	/*
	 * Define I2C Chip addresses
	 */
	#define GEMINI_I2C_IO	B8(01000000)
	#define GEMINI_I2C_AD	B8(10010000)
	#define GEMINI_I2C_TIME B8(10100000)

	/*
	 * Type enum (read / write time)
	 */
	enum geminiTime
	{
		TIME_HS = 1, 	// Time in 1/100s
		TIME_S,			// Time in 1s
		TIME_M,			// Time in 1m
		TIME_H,			// Time in 1h
		TIME_D,			// Time in 1d
	};


	/** 
	 * @brief 	Intializes all peripherials on the Gemini Board
	 * @param 	void
	 * @return 	none
	 */
	void geminiInit();

	/** 
	 * @brief 	Write a value to the 16bit shift register leds
	 * @param 	value Value to write
	 * @return 	none
	 */
	void geminiLedWrite(unsigned int value);

	/** 
	 * @brief 	Write a value to one of the digital potentiometers
	 * @param 	channel	Pot. channel (0 or 1)
	 * @param 	value 	Value to write
	 * @return 	none
	 */
	void geminiPotWrite(unsigned char channel, unsigned char value);

	/** 
	 * @brief 	Read the value of the io extender
	 * @param 	none
	 * @return	Port value
	 */
	unsigned char geminiIoRead();

	/** 
	 * @brief 	Write a value to the io extender
	 * @param 	value 	Value to write
	 * @return 	none
	 */
	void geminiIoWrite(unsigned char value);

	/** 
	 * @brief 	Read the time from the real time clock
	 * @param 	type	Field to be read (TIME_HS for 1/100s, TIME_S for 1s, TIME_M for 1m, TIME, H for 1h) 	
	 * @return 	Time value
	 */
	unsigned char geminiTimeRead(unsigned char type);

	/** 
	 * @brief 	Set the time of the real time clock
	 * @param 	type 	Field to be set (TIME_HS for 1/100s, TIME_S for 1s, TIME_M for 1m, TIME, H for 1h) 		
	 * @param 	value Value to write
	 * @return 	none
	 */
	void geminiTimeWrite(unsigned char type, unsigned char value);

	/** 
	 * @brief 	Read the current voltage from one of the channels of the ad converter
	 * @param 	channel	Channel to be read (0 to 3)
	 * @return 	AD value
	 */
	unsigned char geminiAdRead(unsigned char channel);

	/** 
	 * @brief 	Set output voltage of DA converter
	 * @param 	value	Value for DA conversion	
	 * @return 	none
	 */
	void geminiDaWrite(unsigned char value);

	/** 
	 * @brief 	Set Data Direction Register value of Port A (connected over 16bit io extender)
	 * @param 	value	DDR value	
	 * @return 	none
	 */
	void geminiDDRAWrite(unsigned char value);

	/** 
	 * @brief 	Read Data Direction Register value of Port A (connected over 16bit io extender)
	 * @param	none
	 * @return 	DDR value
	 */
	unsigned char geminiDDRARead();

	/** 
	 * @brief 	Set Port Value of Port A (connected over 16bit io extender)
	 * @param 	value	DDR value	
	 * @return 	none
	 */
	void geminiPORTAWrite(unsigned char value);

	/** 
	 * @brief 	Read Port value of Port A (connected over 16bit io extender)
	 * @param	none
	 * @return 	Port value
	 */
	unsigned char geminiPORTARead();

	/** 
	 * @brief 	Read Pin value of Port A (connected over 16bit io extender)
	 * @param	none
	 * @return 	Pin value
	 */
	unsigned char geminiPINARead();

	/** 
	 * @brief 	Set Data Direction Register value of Port C (connected over 16bit io extender)
	 * @param 	value	DDR value	
	 * @return 	none
	 */
	void geminiDDRCWrite(unsigned char value);

	/** 
	 * @brief 	Read Data Direction Register value of Port C (connected over 16bit io extender)
	 * @param	none
	 * @return 	DDR value
	 */
	unsigned char geminiDDRCRead();

	/** 
	 * @brief 	Set Port Value of Port C (connected over 16bit io extender)
	 * @param 	value	DDR value	
	 * @return 	none
	 */
	void geminiPORTCWrite(unsigned char value);

	/** 
	 * @brief 	Read Port value of Port C (connected over 16bit io extender)
	 * @param	none
	 * @return 	Port value
	 */
	unsigned char geminiPORTCRead();

	/** 
	 * @brief 	Read Pin value of Port C (connected over 16bit io extender)
	 * @param	none
	 * @return 	Pin value
	 */
	unsigned char geminiPINCRead();

#endif
