/*
 * Provide several I2C functions (based on pfleury's lib) 
 *
 * Autor: Bernhard Wrndl-Aichriedler 
 * E-Mail: bwa@berniwa.com 
 * Homepage: http://www.berniwa.com
 */

/**
 * @defgroup blib I2C Library
 *
 * @brief Provide several I2C functions
 *
 * @author Bernhard Wrndl-Aichriedler bwa@berniwa.com http://www.berniwa.com
 */

#ifndef _I2C_H
	#define _I2C_H   1

	/** defines the data direction (reading from I2C device) in i2c_start(),i2c_rep_start() */
	#define I2C_READ    1

	/** defines the data direction (writing to I2C device) in i2c_start(),i2c_rep_start() */
	#define I2C_WRITE   0

	/**
	 * @brief initialize the I2C master interace. Need to be called only once 
	 * @param  void
	 * @return none
	 */
	 void i2cInit(void);


	/** 
	 * @brief Terminates the data transfer and releases the I2C bus 
	 * @param void
	 * @return none
	 */
	 void i2cStop(void);


	/** 
	 * @brief Issues a start condition and sends address and transfer direction 
  
	 * @param    addr address and transfer direction of I2C device
	 * @retval   0   device accessible 
	 * @retval   1   failed to access device 
	 */
	 unsigned char i2cStart(unsigned char addr);


	/**
	 * @brief Issues a repeated start condition and sends address and transfer direction 

	 * @param   addr address and transfer direction of I2C device
	 * @retval  0 device accessible
	 * @retval  1 failed to access device
	 */
	 unsigned char i2cRepStart(unsigned char addr);


	/**
	 * @brief Issues a start condition and sends address and transfer direction 
   
	 * If device is busy, use ack polling to wait until device ready 
	 * @param    addr address and transfer direction of I2C device
	 * @return   none
	 */
	 void i2cStartWait(unsigned char addr);

 
	/**
	 * @brief Send one byte to I2C device
	 * @param    data  byte to be transfered
	 * @retval   0 write successful
	 * @retval   1 write failed
	 */
	 unsigned char i2cWrite(unsigned char data);


	/**
	 * @brief    read one byte from the I2C device, request more data from device 
	 * @return   byte read from I2C device
	 */
	 unsigned char i2cReadAck(void);

	/**
	 * @brief    read one byte from the I2C device, read is followed by a stop condition 
	 * @return   byte read from I2C device
	 */
	 unsigned char i2cReadNak(void);

	/** 
	 * @brief    read one byte from the I2C device
	 * 
	 * Implemented as a macro, which calls either i2c_readAck or i2c_readNak
	 * 
	 * @param    ack 1 send ack, request more data from device<br>
	 *               0 send nak, read is followed by a stop condition 
	 * @return   byte read from I2C device
	 */
	 unsigned char i2cRead(unsigned char ack);

	/** 
	 * @brief    Read a specific register of a i2c slave (start -> transmit reg. -> restart -> read -> stop) 
	 * @param    addr	Address of the I2C device
	 * @param    reg	Register to read
	 * @return   byte read from I2C device, 0 if connection failed
	 */	
	unsigned char i2cRegisterRead(unsigned char addr, unsigned char reg);

	/** 
	 * @brief    Write a specific register of a i2c slave (start -> transmit reg. -> write -> stop) 
	 * @param    addr	Address of the I2C device
	 * @param    reg	Register to write
	 * @param    data	Byte to transfer
	 * @retval   0   device accessible 
	 * @retval   1   failed to access device 
	 */	
	unsigned char i2cRegisterWrite(unsigned char addr, unsigned char reg, unsigned char data);

	/** 
	 * @brief    Read one value from a i2c slave (start -> read -> stop) 
	 * @param    addr	Address of the I2C device
	 * @return   byte read from I2C device, 0 if connection failed
	 */	
	unsigned char i2cDirectRead(unsigned char addr);

	/** 
	 * @brief    Write one value to a i2c slave (start -> write -> stop) 
	 * @param    addr	Address of the I2C device
	 * @param    data	Byte to transfer
	 * @retval   0   device accessible 
	 * @retval   1   failed to access device 
	 */	
	unsigned char i2cDirectWrite(unsigned char addr, unsigned char data);

#endif
