/*
 * Provides several functions for HD44780, KS0073 and PCF2116 (I2C) based LC displays
 * 
 *
 * Autor: Bernhard Wrndl-Aichriedler 
 * E-Mail: bwa@berniwa.com 
 * Homepage: http://www.berniwa.com
 */

/**
 * @defgroup blib LCD Library
 *
 * @brief Provides several functions for HD44780, KS0073 and PCF2116 (I2C) based LC displays
 *
 * @author Bernhard Wrndl-Aichriedler bwa@berniwa.com http://www.berniwa.com
 */


#ifndef _LCD_H
	#include "lcd.h"
	#include "default.h"
	#include <stdlib.h>
	#include <string.h>

	#if LCD_MODE == LCD_MODE_I2C

		#include "i2c.h"

	/**
	 * Initialize LCD
	 */
	void lcdInit()
	{
		if(!i2cStart(I2C_ADDRESS_LCD+I2C_WRITE))
	    {
			i2cWrite(B8(00000000)); //Set mode
			i2cWrite(B8(00101110)); //Select a 4 line display, 
			i2cWrite(B8(00001110)); //Enable display
			i2cWrite(B8(00000110));
			i2cStop();
		}

	}

	/**
	 * Set cursor position
	 *
	 * @param char line Line no (starting with 1)
	 * @param char element Charakter index 
	 */
	void lcdPosition(unsigned char line, unsigned char element)
	{
		if(!i2cStart(I2C_ADDRESS_LCD+I2C_WRITE))
	    {
			i2cWrite(B8(10000000));
		
			unsigned char address = (line - 1)*0x20 + element; 

			i2cWrite(address | 128);
		
			i2cStop();
		}
	
	}

	/**
	 * Write string to display
	 *
	 * @param char *data String to write
	 */
	void lcdWrite(char *data)
	{
		if(!i2cStart(I2C_ADDRESS_LCD+I2C_WRITE))
	    {
			i2cWrite(B8(01000000)); //Set write
		
			unsigned char c;

		    while ( (c = *data++) ) 
			{
				if(c + LCD_DIRECTION < 255)
				{
					i2cWrite(c + LCD_DIRECTION);
				}
		    }
		
			i2cStop();
		}
	}

	/*
	 * Write a single sign - binary codes -> see docu
	 * 
	 * @param char data Sign to write
	 */ 
	void lcdWriteDirect(unsigned char data)
	{
		if(!i2cStart(I2C_ADDRESS_LCD+I2C_WRITE))
	    {
			i2cWrite(B8(01000000)); //Set write
			i2cWrite(data);
			i2cStop();
		}
	}

	/**
	 * Write a int value to the display
	 *
	 * @param int value Value to write
	 */
	void lcdWriteInt(int value)
	{
		char data[6];
		itoa(value,data,10);
		lcdWrite(data);
	}

	/**
	 * Clear LCD
	 */
	void lcdClear(void)
	{
		if(!i2cStart(I2C_ADDRESS_LCD+I2C_WRITE))
	    {
			i2cWrite(B8(10000000)); //Set control
			i2cWrite(B8(00000001)); //Clear display
			i2cStop();
			mSleep(1); //Wait 1ms to clear display - see datasheet
		}

	}
	#elif(LCD_MODE == LCD_MODE_HD44780) || (LCD_MODE == LCD_MODE_KS0073)


	#include "fleury/fleury_lcd.h"


	/**
	 * Initialize LCD
	 */
	void lcdInit(){lcd_init(LCD_DISP_ON);}

	/**
	 * Set cursor position
	 *
	 * @param char line Line no (starting with 1)
	 * @param char element Charakter index 
	 */
	void lcdPosition(unsigned char line, unsigned char element){lcd_gotoxy(element, line-1);}

	/**
	 * Write string to display
	 *
	 * @param char *data String to write
	 */
	void lcdWrite(char *data){lcd_puts(data);}

	/*
	 * Write a single sign - binary codes -> see docu
	 * 
	 * @param char data Sign to write
	 */ 
	void lcdWriteDirect(unsigned char data){lcd_data(data);}

	/**
	 * Write a int value to the display
	 *
	 * @param int value Value to write
	 */
	void lcdWriteInt(int value)
	{
		char data[6];
		itoa(value,data,10);
		lcdWrite(data);
	}

	/**
	 * Clear LCD
	 */
	void lcdClear(void){lcd_clrscr();}	


	#else 
		#error "DISPLAY NOT SUPPORTED"

	#endif

#endif
