/*
 * Provides several functions for Timer 1 based PWM generation
 * 
 *
 * Autor: Bernhard Wrndl-Aichriedler 
 * E-Mail: bwa@berniwa.com 
 * Homepage: http://www.berniwa.com
 */

/**
 * @defgroup blib PWM Library
 *
 * Provides several functions for Timer 1 based PWM generation
 *
 * @author Bernhard Wrndl-Aichriedler bwa@berniwa.com http://www.berniwa.com
 */

#ifndef _PWM_H
	#include "pwm.h"
	#include "default.h"
	#include <avr/io.h>

	unsigned int pwm_prescaler = 0;
	unsigned int pwm_top = 0x00FF;

	/**
	 * Init and start phase frequency correct PWM output 
	 *
	 * @param uchar port Enabled PWM Ports (1 for OC1A, 2 for OC1B, 3 for both)
	 * @param uchar bits This value is used set the top counter value (8 bit TOP = 0x00FF, 9 bit TOP = 0x01FF, 10 bit TOP = 0x03FF)
	 * @param uint prescaler Clock prescaler (1, 8, 64, 256, 1024)
	 * @param uchar percentage Initial PWM output percentage (0 - 100%), for all enabled ports
	 */
	void pwmInit(unsigned char port, unsigned char bits, unsigned int prescaler, unsigned int percentage)
	{
		if(port > 3){port = 3;}

		DDRB |= (port<<PB1);

		TCCR1A = 0;

		if(bits == 8)
		{
			TCCR1A |= (1<<WGM10);
			pwm_top = 0x00ff;
		}
		else if(bits == 9)
		{
			TCCR1A |= (1<<WGM11)|(0<<WGM10);
			pwm_top = 0x01ff;
		}
		else if(bits == 10)
		{
			TCCR1A |= (1<<WGM11)|(1<<WGM10);
			pwm_top = 0x03ff;
		}

		if((port & B8(00000010)))
		{
			TCCR1A |= (1<<COM1B1);
		}
		if((port & B8(00000001)))
		{
			TCCR1A |= (1<<COM1A1);
		}

		pwmPrescaler(prescaler);
		pwmPercentage(port, percentage);
	}

	/**
	 * Start PWM for all enabled pins, use this to reenable the PWM after stopping it with pwmStop()
	 */
	void pwmStart()
	{
		pwmPrescaler(pwm_prescaler);
	}

	/**
	 * Stop PWM for all enabled pins
	 */
	void pwmStop()
	{
		TCCR1B &= B8(11111000); //Disable clock
	}

	/**
	 * Set the PWM output percentage
	 *  
	 * @param uchar port Ports (1 for OC1A, 2 for OC1B, 3 for both)
	 * @param uchar percentage Output percentage (0 - 100%)
	 */
	void pwmPercentage(unsigned char port, unsigned char percentage)
	{
		if((port & B8(00000010)))
		{
			OCR1B = (int)(pwm_top * percentage / 100);			
		}
		if((port & B8(00000001)))
		{
			OCR1A = (int)(pwm_top * percentage / 100);	
		}

	}

	/**
	 * Set the clock prescaler 
	 * The lower you set this value the more switching operation are performed
	 * 
	 * @param unit prescaler New prescaler (1, 8, 64, 256, 1024)
	 */
	void pwmPrescaler(unsigned int prescaler)
	{
		unsigned char cs = 0;

		if(prescaler == 1)
		{
			cs = (1<<CS10);
		}
		else if(prescaler == 8)
		{
			cs = (2<<CS10);
		}
		else if(prescaler == 64)
		{
			cs = (3<<CS10);
		}
		else if(prescaler == 256)
		{
			cs = (4<<CS10);
		}
		else if(prescaler == 1024)
		{
			cs = (5<<CS10);
		}
		else
		{
			prescaler = 256; //Default prescaler 256
			cs = (4<<CS10);
		}

		TCCR1B &= B8(11111000);
		TCCR1B |= cs;

		pwm_prescaler = prescaler;
	}

#endif
