/*******************************************************************
	Module: ringbuffer

*******************************************************************/
#include "ringbuffer.h"
#include "default.h"

#define RINGBUFFER_DATA_SIZE 32
#define RINGBUFFER_NO_MAX    2

#define RINGBUFFER_FLAG_OVERFLOW 0x02

typedef struct
{
	unsigned char Read;
	unsigned char Write;
	unsigned char Flags;
	unsigned char Buffer[RINGBUFFER_DATA_SIZE];

} TRingbufferStruct;

typedef struct 
{
	TRingbufferStruct ringbuffer[RINGBUFFER_NO_MAX];
	unsigned char Occupied[RINGBUFFER_NO_MAX];
	unsigned int NoOfringbuffers;
} TBufferPool;

TBufferPool TRingbufferPool;

/*******************************************************************
	Function: ringbufferInit
	Description:
		Initialise a number of ringbuffers. This defines the maximum
		simultanious used ringbuffers.
	Return:
		true if the wanted number of ringbuffer can be
		initialised, otherwise C51_FALSE.

*******************************************************************/
unsigned char ringbufferInit( unsigned int aNoOfringbuffers )
{
	unsigned int index;
	
	for (index = 0;index<aNoOfringbuffers;index++)
	{
		TRingbufferPool.Occupied[index] = 0;
	}
	TRingbufferPool.NoOfringbuffers = aNoOfringbuffers;

	return true;
}

/*******************************************************************
	Function: ringbufferInit
	Description:
		Free the memory. The created ringbuffers must not be used any more!!
	Return:
		true if the wanted number of ringbuffer can be
		initialised, otherwise C51_FALSE.

 *******************************************************************/
unsigned char ringbufferDone( void )
{
	unsigned int index;

	for (index = 0;index<TRingbufferPool.NoOfringbuffers;index++)
	{
		TRingbufferPool.Occupied[index] = 0;
	}
	TRingbufferPool.NoOfringbuffers=0;

	return true;
}

/*******************************************************************
	Function: ringbufferCreate
	Description:
		Create a new ringbuffer
	Return:
		TRingbuffer Created ringbuffer
*******************************************************************/
TRingbuffer ringbufferCreate( void )
{
	unsigned int index;
	TRingbufferStruct *ringbuffer;
	//search a free ringbuffer
	for (index = 0;index < TRingbufferPool.NoOfringbuffers; index++)
	{
		if (TRingbufferPool.Occupied[index] == 0)
		{
			break;
		}
	}

	if (index >= TRingbufferPool.NoOfringbuffers)
		return 0;


	//reserve the ringbuffer
	TRingbufferPool.Occupied[index] = 1;

	//init the ringbuffer
	ringbuffer = &TRingbufferPool.ringbuffer[index];

	ringbuffer->Read = 0;
	ringbuffer->Write = 0;
	ringbuffer->Flags = 0;

	return index;
}


/*******************************************************************
	Function: ringbufferDestroy
	Description:
		Destroy a single ringbuffer
*******************************************************************/
unsigned char ringbufferDestroy( TRingbuffer aringbuffer )
{
	unsigned char index = aringbuffer;
	if(index > 0 && index < TRingbufferPool.NoOfringbuffers)
	{
		TRingbufferPool.Occupied[index] = 0;

		return true;
	}
	else
	{
		return false;
	}
}

/*******************************************************************
	Function: ringbufferWrite
	Description:
		Write a databyte into the ringbuffer
	Return:
		true, when write was successfull
*******************************************************************/
unsigned char ringbufferWrite( TRingbuffer aringbuffer, unsigned char aDataByte)
{
	unsigned char write;

	if(aringbuffer >= 0 && aringbuffer < TRingbufferPool.NoOfringbuffers)
	{
		TRingbufferStruct *ringbuffer;
		ringbuffer = &TRingbufferPool.ringbuffer[aringbuffer];
		
		write = (ringbuffer->Write+1)%RINGBUFFER_DATA_SIZE;

		if(write != ringbuffer->Read)
		{
			ringbuffer->Buffer[write] = aDataByte;
			ringbuffer->Write = write;
		}
		else
		{
			ringbuffer->Flags = RINGBUFFER_FLAG_OVERFLOW;
			return false;
		}
		return true;
	}
	else
	{
		return true;
	}
}

/*******************************************************************
	Function: ringbufferRead
	Description:
		Read a byte out of the ringbuffer
	Return:
		RINGBUFFER_EMPTY, ringbuffer is empty
		RINGBUFFER_SUCCESS, Read successfull
		RINGBUFFER_OVERFLOW, ringbuffer overflow detected
*******************************************************************/
TRingbufferState ringbufferRead(TRingbuffer aringbuffer, unsigned char *aDataByte)
{
	unsigned char read;

	if(aringbuffer >= 0 && aringbuffer < TRingbufferPool.NoOfringbuffers)
	{
		TRingbufferStruct *ringbuffer;
		ringbuffer = &TRingbufferPool.ringbuffer[aringbuffer];

		if(ringbuffer->Read != ringbuffer->Write)
		{	
			read = (ringbuffer->Read+1)%RINGBUFFER_DATA_SIZE;
			*aDataByte = ringbuffer->Buffer[read];
			ringbuffer->Read = read;
			if(ringbuffer->Flags == RINGBUFFER_FLAG_OVERFLOW)
			{
				return RINGBUFFER_OVERFLOW;
			}
			else
			{
				return RINGBUFFER_SUCCESS; 
			}
		}
		else
		{
			ringbuffer->Flags = 0;
			return RINGBUFFER_EMPTY;
		}
	}
	else
	{
		return false;
	}
}
