/*
 * Driver for the Samsung KS0713 controller built into 
 * the CFax 128 x 64 graphical LCD with touch screen from CrystalFontz
 * 
 * see http://www.sparkfun.com/commerce/product_info.php?products_id=257
 * for further information
 * 
 * The library is ported from the LPC2138 ARM sample from sparkfun.com to
 * the ATMega32 of Atmels AVR series.
 *
 * Modify the glcdOut, glcdRs, glcdRd, glcdRw & glcdRstb routines to suit your hardware
 *
 * Author: Bernhard Wrndl-Aichriedler @ project-xmedia.com
 * E-Mail:bwa@berniwa.com
 * Homepage: www.berniwa.com - www.project-xmedia.com
 *
 * @author Bernhard Wrndl-Aichriedler
 */


#ifndef _GLCD_H

	#define _GLCD_H

	/* 
	 * PORT, PIN, DDR definition
	 * for the 8bit wide data port
	 */
	#define GLCD_DDR_DATA DDRA
	#define GLCD_PORT_DATA PORTA
	#define GLCD_PIN_DATA PINA

	/* 
	 * PORT, PIN, DDR definition
	 * for the RD line of the display
	 */
	#define GLCD_PORT_RD PORTC
	#define GLCD_DDR_RD DDRC
	#define GLCD_PIN_RD PC5

	/* 
	 * PORT, PIN, DDR definition
	 * for the RW line of the display
	 */
	#define GLCD_PORT_RW PORTC
	#define GLCD_DDR_RW DDRC
	#define GLCD_PIN_RW PC4

	/* 
	 * PORT, PIN, DDR definition
	 * for the RS line of the display
	 */
	#define GLCD_PORT_RS PORTC
	#define GLCD_DDR_RS DDRC
	#define GLCD_PIN_RS PC7

	/* 
	 * PORT, PIN, DDR definition
	 * for the RST line of the display
	 */
	#define GLCD_PORT_RST PORTG
	#define GLCD_DDR_RST DDRG
	#define GLCD_PIN_RST PG2

	/*
	 * Initialize Graphic LCD
	 */
	void glcdInit();

	/*
	 * Write a character string to the display
	 * @param char* line Sting to write (zero - terminated)
	 * @param char x Page to write to (0 - 8)
	 * @param char y Start element (0 - 127)
	 */
	void glcdWrite(char *line, unsigned char x, unsigned char y);

	/*
	 * Write a single character to the display
	 * @param uchar c char to write
	 * @param char x Page to write to (0 - 8)
	 * @param char y Start element (0 - 127)
	 */
	void glcdPutChar(unsigned char c, unsigned char x, unsigned char y);

	/*
	 * Write raw pixel data to the display
	 * @param uchar data char to write
	 * @param char x Page to write to (0 - 8)
	 * @param char y Start element (0 - 127)
	 */
	void glcdWriteData(unsigned char data, unsigned char page, unsigned char column);

	/*
	 * Write a line of raw pixel data to the display
	 * @param uchar *data Raw data to write (zero - terminated)
	 * @param char page Page to write to (0 - 8)
	 * @param char column Start element (0 - 127)
	 */
	void glcdWriteDataLine(char *data, unsigned char length, unsigned char page, unsigned char column);

	/*
	 * Draw a single pixel
	 * @param uchar value Set / Clear Pixel (0 / 1)
	 * @param char x Pixel to write to (0 - 64)
	 * @param char y Start element (0 - 127)
	 */
	void glcdDrawPixel(unsigned char value, unsigned char x, unsigned char y);

	/*
	 * Execute a control command of the lcd
	 * @param uchar command Control byte to send to the display
	 */
	void glcdCommand(unsigned char command);

	/*
	 * Clear entire screen
	 */
	void glcdClearScreen(void);

	/*
	 * Set/clear RD line
	 * @param stat PIN State (0 / 1)
	 */	
	void glcdRd(char stat);

	/*
	 * Set/clear RW line
	 * @param stat PIN State (0 / 1)
	 */	
	void glcdRw(char stat);

	/*
	 * Set/clear RS line
	 * @param stat PIN State (0 / 1)
	 */	
	void glcdRs(char stat);

	/*
	 * Set/clear RSTB line
	 * @param stat PIN State (0 / 1)
	 */	
	void glcdRstb(char stat);

	/*
	 * Write to the dataport
	 * @param uchar cmd Byte to be written
	 */
	void glcdOut(unsigned char cmd);

	/*
	 * Read from dataport
	 * @return uchar Read data
	 */
	unsigned char glcdIn();

	/*
	 * Delay a 100us (may be removed in later versions)
	 */
	void glcdDelay(void);

	/*
	 * Set PORT as Output
	 */
	void glcdSetAsOutput();

	/*
	 * Set PORT as Input
	 */
	void glcdSetAsInput();


#endif
