/*============================================================================
    Project     QT1106 Demo Code
    File        qt1106.c
    Function    Driver code for QT1106 - QWheel, QSlide, QTouch IC
    Requires    Silicon Labs C8051F310 target
    Originated	D Spokes
    
    (c) Quantum Research Group
============================================================================*/
#include "QT1106.h"
#include "../blib/spi.h"
#include "../blib/default.h"
#include "capInterface.h"
#include <avr/interrupt.h>

/* DEFINES */

#define DELAY_30US				30
#define DELAY_200US				200
#define POLL_ON_CHANGE_HOLDOFF	10 

enum {	/* QT Settings - continues enum declared in qt1106.h
           these values are not defined outside this module */
	SET_QT_CAL_WHEEL = SET_QT_RESOLUTION + 1,
	SET_QT_CAL_KEY
	};

#define NUM_SPI_BYTES			3			/* number of bytes in SPI message */
#define NUM_COMMAND_FIELDS		10			/* number of command fields in SPI message */
#define CT_BIT					0x80		/* bit 7 of byte 1 of SPI command message */

#define SET_QT_PROX_MASK		0x10
#define SET_QT_SLD_MASK			0x08
#define SET_QT_AKS_MASK			0x07
#define SET_QT_MOD_MASK			0x60
#define SET_QT_DI_MASK			0x10
#define SET_QT_LPB_MASK			0x08
#define SET_QT_MODE_MASK		0x07
#define SET_QT_RESOLUTION_MASK	0xe0
#define SET_QT_CAL_WHEEL_MASK	0x10
#define SET_QT_CAL_KEY_MASK		0x08
#define SET_QT_CAL_KEY_NUM_MASK	0x07


/*	************ TARGET DEPENDENT CODE *************	*/
#define QT_DRDY_IN PINB & (1 << PB4)
#define QT_CHANGE_IN PINB & (1 << PB5)
#define QT_SS_HIGH() PORTB |= (1 << PB0)
#define QT_SS_LOW() PORTB &= ~(1 << PB0)


/* LOCAL DATA */

/* SPI comms buffers */
unsigned char QtCommand[NUM_SPI_BYTES];	/* normal command message ( CT = 0 ) */
unsigned char QtThreshold[NUM_SPI_BYTES];	/* 'Set threshold' message ( CT = 1 ) */
unsigned char QtResponse[NUM_SPI_BYTES];	/* message returned by QT1106 */

/* current Command settings are stored in an array */
unsigned char QtCommandField[NUM_COMMAND_FIELDS];

/* driver flags */
unsigned char SsWakeFlag;
unsigned char DoQtComms;


/* driver comms setings */
unsigned int QtPollTime;	/* time-stamp */
unsigned int QtPollRate;	/* time in milliseconds between polls */
unsigned char QtCommsMode;	/* communication mode */
 
/* bitfield-masks used to add command fields to MOSI message */
unsigned char QtCommandFieldMask[] = {
									SET_QT_PROX_MASK,
								 	SET_QT_SLD_MASK,
								 	SET_QT_AKS_MASK,
								 	SET_QT_MOD_MASK,
								 	SET_QT_DI_MASK,
								 	SET_QT_LPB_MASK,
								 	SET_QT_MODE_MASK,
								 	SET_QT_RESOLUTION_MASK,
								 	SET_QT_CAL_WHEEL_MASK,
									SET_QT_CAL_KEY_MASK | SET_QT_CAL_KEY_NUM_MASK
									};

/* MODULE FUNCTIONS */
void DelayUs ( unsigned int Delay );
unsigned char QtComms ( void );
/*----------------------------------------------------------------------------*/



/*---------------- Function Header -------------------------------------------

	FUNCTION	InitQt1106
	PURPOSE		Initialises QT1106 driver for subsequent processing

----------------------------------------------------------------------------*/
void InitQt1106 ( void )
{
	DDRB = B8(00000111);
	PORTB = B8(00000001);

	DDRG |= (1 << PG3);
	PORTG |= (1 << PG3);

	PCICR |= (1 << PCIE0);
	PCIFR |= (1 << PCIF0);
	PCMSK0 |= (1 << PCINT5); //Only sense on change pin

	spiMasterInit();
	

/* ---- Initialise MOSI command bytes ------------------ */
	QtCommand[0] 	= 0;
	QtCommand[1] 	= 0;
	QtCommand[2] 	= 0;

	QtThreshold[0] 	= 0;
	QtThreshold[1] 	= 0;
	QtThreshold[2] 	= 0;

/* ---- Setup initial QT command bytes as required ----- */
	/* . The example here sets the QT1106 default settings */
	SetQtCommandField ( SET_QT_PROX, 		QT_PROX_OFF		);
	SetQtCommandField ( SET_QT_SLD,  		QT_MODE_WHEEL	);
	SetQtCommandField ( SET_QT_AKS,  		QT_AKS_DISABLED	);
	SetQtCommandField ( SET_QT_MOD,  		QT_MOD_10S		);
	SetQtCommandField ( SET_QT_DI,   		QT_DI_6_DETECTS	);
	SetQtCommandField ( SET_QT_LPB,  		QT_LPB_ON		);
	SetQtCommandField ( SET_QT_MODE, 		QT_MODE_FREE_RUN);
	SetQtCommandField ( SET_QT_RESOLUTION, 	QT_RESOLUTION_7	);

/* ---- Clear QT response array ----------------------- */
	QtResponse[0] = 0;
	QtResponse[1] = 0;
	QtResponse[2] = 0;
	
/* ---- Initialise driver flags ----------------------- */
	/* /SS wakeup is selected because QT1106 mode is unknown */
	SsWakeFlag = true; 
	DoQtComms = false;

}


/*---------------- Function Header -------------------------------------------

	FUNCTION	QtComms
	PURPOSE		Tx/Rx via hardware SPI

Notes:
	1. The SPI is only executed if DRDY is sensed high
	2. A /SS wakeup pulse is output if the last Mode sent to the QT was other
	   than free-run mode (SsWakeFlag is the control)
	3. The function terminates if DRDY does not acknowledge the /SS wake pulse
	4. Either a normal command or a 'set threshold' command will be sent, depending
	   on the state of the CT bit in the QtThreshold[] array
    5. If the above bit is set, it is automatically cleared after the threshold has been sent
	6. CALW and CALK bits are automatically cleared in QtCommand[] after being sent

	************ TARGET DEPENDENT CODE *************
	This function manipulates bits within C8051F310 control registers

	INPUT:		None
	OUTPUT:		Flag indicating if a SPI cycle was executed

----------------------------------------------------------------------------*/
unsigned char QtComms ( void )
{
	unsigned char i;
	unsigned char *TxPtr;
	unsigned int Timeout = 1000;
					
	if ( QT_DRDY_IN )				/* can not execute SPI while DRDY is low */
	{
		if ( SsWakeFlag )			/* generate /SS Wake pulse if required */
		{
			QT_SS_LOW();				/* start /SS Wake pulse */
			DelayUs (DELAY_200US);
			QT_SS_HIGH();			/* end /SS Wake pulse */			
		
			while ( QT_DRDY_IN && Timeout ) Timeout--;	/* wait for start of low pulse on DRDY */
			while ( !QT_DRDY_IN && Timeout ) Timeout--;	/* wait for end of low pulse on DRDY */
		}
	}
	else
	{
		Timeout = 0;				/* Timeout is used as abort flag */
	}

	/* check valid start condition has been established */
	if ( !Timeout )
	{
		return false;				/* and exit function */
	}		

	/* initiate transfer */
	QT_SS_LOW();						/* set /SS low */
 
	DelayUs (DELAY_30US);			/* wait for >22us */

	/* check which message to send (defined by CT bit in QtThreshold[0]) */
	if ( QtThreshold[0] )
		TxPtr = QtThreshold;
	else
		TxPtr = QtCommand;

	for (i = 0; i < NUM_SPI_BYTES; i++) /* send three bytes */
	{
		QtResponse[i] = spiMasterWrite(*TxPtr++);
	}
	
	QT_SS_HIGH();					/* set /SS high to end serial transfer */

	/* post processing - executed after a successful SPI cycle */
	if ( QtThreshold[0] )
		QtThreshold[0] = 0; 		/* clear CT if threshold was sent */
	else
	{
		SetQtCommandField ( SET_QT_CAL_WHEEL, 0 ); /* clear calibration command bits */
		SetQtCommandField ( SET_QT_CAL_KEY, 0 );
		
		/* set SsWake flag depending on mode just commanded */
		SsWakeFlag = (QtCommandField[ SET_QT_MODE ] == QT_MODE_FREE_RUN) ? false : true;
	}

	return true;

}

/*---------------- Function Header -------------------------------------------

	FUNCTION	DelayUs
	PURPOSE		Adds fixed execution delay in the microsecond range
	INPUT		Value representing the required delay			

----------------------------------------------------------------------------*/
void DelayUs ( unsigned int Delay )
{
	uSleep(Delay);
}


/*---------------- Function Header -------------------------------------------

	FUNCTION	GetQtCommandField
	PURPOSE		Helper function, which returns the current setting of the
	            requested command field
	INPUTS:		Field: defines command field to return
	OUTPUTS:	Returns setup value (only if Field is valid) 

----------------------------------------------------------------------------*/
unsigned char GetQtCommandField ( unsigned char Field )
{
	if ( Field < NUM_COMMAND_FIELDS )
		return QtCommandField[ Field ];
	else
		return 0;
}

/*---------------- Function Header -------------------------------------------

	FUNCTION	GetQtStatus
	PURPOSE		Helper function, which returns the requested byte of the last
	            QT1106 response on MISO or Custom threshold
	INPUTS:		Field: defines response field to return
	OUTPUTS:	Returns requested field

----------------------------------------------------------------------------*/
unsigned char GetQtStatus ( unsigned char Field )
{
	switch ( Field )
	{
	case QT_FLAG_STATE:
	case QT_KEY_STATE:
	case QT_WHEEL_POSITION:		return QtResponse[ Field ];
	case QT_THRESHOLD:			return QtThreshold[ 1 ];
	default:					return 0;
	}
}

/*---------------- Function Header -------------------------------------------

	FUNCTION	SetQtCommandField
	PURPOSE		Writes the specified command field to the SPI command message,
	            which is maintained in QtCommand[]. The value is also copied to
				QTCommandField[] for use by the driver.
				After any field is written, DoQtComms is set. This causes the driver
				to write the new command to the QT1106 as soon as DRDY is sensed high.
				
	INPUTS:		Field: defines command field to write
				Value: new value for specified field
	OUTPUTS:	None

-----------------------------------------------------------------------------*/
void SetQtCommandField ( unsigned char Field, unsigned char Value )
{

	switch ( Field )
	{
	case SET_QT_PROX:	/* process fields in first SPI byte */
	case SET_QT_SLD:
	case SET_QT_AKS:
		QtCommandField[ Field ] = Value & QtCommandFieldMask[ Field ];
		QtCommand[0] &= ~QtCommandFieldMask[ Field ];
		QtCommand[0] |= QtCommandField[ Field ];
		DoQtComms = true;
		break;
	case SET_QT_MOD:	/* process fields in second SPI byte */
	case SET_QT_DI:
	case SET_QT_LPB:
	case SET_QT_MODE:
		QtCommandField[ Field ] = Value & QtCommandFieldMask[ Field ];
		QtCommand[1] &= ~QtCommandFieldMask[ Field ];
		QtCommand[1] |= QtCommandField[ Field ];
		DoQtComms = true;
		break;
	case SET_QT_RESOLUTION:	/* process fields in last SPI byte */
	case SET_QT_CAL_WHEEL:
	case SET_QT_CAL_KEY:
		QtCommandField[ Field ] = Value & QtCommandFieldMask[ Field ];
		QtCommand[2] &= ~QtCommandFieldMask[ Field ];
		QtCommand[2] |= QtCommandField[ Field ];
		DoQtComms = true;
		break;
	}
}

/*---------------- Function Header -------------------------------------------

	FUNCTION	SetQt1106CommsMode
	PURPOSE		Sets communications method and asynchronous poll-rate
				
	INPUTS:		Mode: defines comms method
				PollRate: defines new poll-rate
	OUTPUTS:	None

-----------------------------------------------------------------------------*/
void SetQt1106CommsMode ( unsigned char Mode, unsigned int PollRate )
{
	if ( Mode <= QT_COMMS_MODE_POLL_ON_CHANGE )
	{
		QtCommsMode = Mode;
		QtPollRate = PollRate;
	}
}

/*---------------- Function Header -------------------------------------------

	FUNCTION	ProcessQt1106
	PURPOSE		Sequences SPI cycles with QT1106
	INPUT		None
	OUTPUT		Returns a flag indicating whether an SPI cycle was executed
	MODIFIES	QT1101KeyState modified by DoQtComms() function
	USE			Called periodically by control programme to update keystate
----------------------------------------------------------------------------*/
unsigned char ProcessQt1106( void )
{
	unsigned char RetVal = false;	/* set default return status (over-written by successful SPI comms cycle */
				 			
/*  DoQtComms is set to activate an SPI cycle depending on mode.

Note: DoQtComms is also set by SetQtCommandField() and CommandQt() to force mode changes and
      Cal and Threshold commands out to the QT1106 */

/* ---- test if it's time to run a comms cycle -------- */
	switch (QtCommsMode)	/* depending on selected mode */
	{
	case QT_COMMS_MODE_POLL_ALWAYS:
		/* do an SPI cycle every time DRDY goes high */
		DoQtComms = true;
		break;
			
	case QT_COMMS_MODE_POLL_ASYNC:
		/* do an SPI cycle at a rate defined by QtPollRate */
		//if (CheckTimer(QtPollTime, QtPollRate)) /* check for poll timeout */
		//{
		//	QtPollTime = StartTimer(); 		/* restart timer */
		//	DoQtComms = true;
		//}
		break;

	case QT_COMMS_MODE_POLL_ON_CHANGE:
		/* do an SPI cycle whenever CHANGE is high - the CheckTimer function prevents multiple SPI cycles*/
		//if ( QT_CHANGE_IN && CheckTimer(QtPollTime, POLL_ON_CHANGE_HOLDOFF))
		//{
		//	QtPollTime = StartTimer(); 		/* restart timer */
		//	/* run comms if CHANGE pin is high */
		//	DoQtComms = true;
		//}
		break;

	}; /* switch (QtCommsMode) */
/* ---------------------------------------------------- */

	if ( DoQtComms ) 
	{
		RetVal = QtComms();		/* attempt to run a comms cycle */
		DoQtComms = !RetVal;	/* and clear DoQtComms if cycle was executed */
	}

	return ( RetVal );
}
/* --------------------------------------------------------------------------- */

/*---------------- Function Header -------------------------------------------

	FUNCTION	CommandQt
	PURPOSE		Applies command to QT via SPI
	INPUTS:		Command: defines command to be actioned
				Arg:	 defines any values required by the command
	OUTPUTS:	None
	----------------------------------------------------------------------------*/
void CommandQt ( unsigned char Command, unsigned char Arg )
{
	switch ( Command )
	{
	case QT_COMMAND_CAL_WHEEL: /* Arg is ignored */
		SetQtCommandField( SET_QT_CAL_WHEEL, SET_QT_CAL_WHEEL_MASK );
		DoQtComms = true;
		break;
	case QT_COMMAND_CAL_KEY: /* Arg specifies the Cal Key Num field as defined in the datasheet */
		SetQtCommandField( SET_QT_CAL_KEY, SET_QT_CAL_KEY_MASK | (Arg & SET_QT_CAL_KEY_NUM_MASK) );
		DoQtComms = true;
		break;
	case QT_COMMAND_SET_THRESHOLD: /* Arg specifies the new threshold value */
		QtThreshold[0] = CT_BIT;
		QtThreshold[1] = Arg;
		QtThreshold[2] = 0;
		DoQtComms = true;
		break;
	}
}


SIGNAL(SIG_PIN_CHANGE0)
{
	if((PINB & (1 << PB5)))
	{
		//Change happened -> wait for data ready
		PCMSK0 |= (1 << PCINT4);
	}


	if((PINB & (1 << PB4)) && (PINB & (1 << PB5))) //if DRDY is 1 and CHANGE is high -> aquire new data
	{
		QtComms();
		capInterfaceMain();

		if(!(PINB & (1 << PB5)))
		{
			PCMSK0 &= ~(1 << PCINT4);
		}
	}
}
