/*
 * Xdi Math Calculation Lib and gestrue sensing
 *
 * Autor: Bernhard Wrndl-Aichriedler 
 * E-Mail: bwa@berniwa.com 
 * Homepage: http://www.berniwa.com
 */

/**
 * @defgroup xdi Math Calculation Lib and gestrue sensing
 *
 * @brief XDI 
 *
 * @author Bernhard Wrndl-Aichriedler bwa@berniwa.com http://www.berniwa.com
 */

#include <math.h>
#include "xdi.h"
#include "../sie/sie.h"
#include "irsensor.h"
#include "../blib/rs232.h"
#include <avr/interrupt.h>


void (*xdi_ui_callback)(unsigned char) = 0;
void (*xdi_ui_position_callback)(unsigned int, unsigned int, unsigned char) = 0;




void xdiInit(void (*ui_callback)(unsigned char), void (*ui_position_callback)(unsigned int, unsigned int, unsigned char))
{
	xdi_ui_callback = ui_callback;
	xdi_ui_position_callback = ui_position_callback;

	irsInit();	
}




/*
 Command Mode
 - Start if tracking > 2 points
 - Capture first coordinate
 - End Capture if tracking < 2 points
 - Return if distance > 2 * pointer difference

 Pointer Mode
- Calibration
-- Wait for 2 Points to appear
-- Track left top point
-- Wait for only 1 Point
-- Wait for 2 Points to appear
-- Track right bottom point
-- Wait for only 1 Point
-- Set Screen values
- Position
-- Wait for 1 Point to appear
-- Move cursor
-- Wait for 2. point
-- Click
*/


typedef enum
{
	XDI_PSTATE_CALIBRATION_WAIT_FOR_TOP_LEFT_2P,
	XDI_PSTATE_CALIBRATION_WAIT_FOR_TOP_LEFT_1P,
	XDI_PSTATE_CALIBRATION_WAIT_FOR_BOTTOM_RIGHT_2P,
	XDI_PSTATE_CALIBRATION_WAIT_FOR_BOTTOM_RIGHT_1P,
	XDI_PSTATE_TRACKING
} XdiPositionStates;

XdiPositionStates xdi_position_state = XDI_PSTATE_TRACKING;
XdiPoint xdi_position_left_top;
XdiPoint xdi_gesture_right_bottom;


void xdiPositionProcessor(XdiPoint p1, XdiPoint p2)
{
	switch(xdi_position_state)
	{
		case XDI_PSTATE_CALIBRATION_WAIT_FOR_TOP_LEFT_2P:
		{
			if(p1.x != XDI_POINT_NOT_TRACKED && p2.x != XDI_POINT_NOT_TRACKED)
			{
				xdi_position_left_top.x = p1.x;
				xdi_position_left_top.y = p1.y;

				xdi_position_state = XDI_PSTATE_CALIBRATION_WAIT_FOR_TOP_LEFT_1P;
			}		
			break;
		}
		case XDI_PSTATE_CALIBRATION_WAIT_FOR_TOP_LEFT_1P:
		{
			if(p1.x == XDI_POINT_NOT_TRACKED || p2.x == XDI_POINT_NOT_TRACKED)
			{
				xdi_position_state = XDI_PSTATE_CALIBRATION_WAIT_FOR_BOTTOM_RIGHT_2P;
			}		
			break;
		}
		case XDI_PSTATE_CALIBRATION_WAIT_FOR_BOTTOM_RIGHT_2P:
		{
			if(p1.x != XDI_POINT_NOT_TRACKED && p2.x != XDI_POINT_NOT_TRACKED)
			{
				xdi_gesture_right_bottom.x = p1.x;
				xdi_gesture_right_bottom.y = p1.y;

				xdi_position_state = XDI_PSTATE_CALIBRATION_WAIT_FOR_TOP_LEFT_1P;
			}		
			break;
		}
		case XDI_PSTATE_CALIBRATION_WAIT_FOR_BOTTOM_RIGHT_1P:
		{
			if(p1.x == XDI_POINT_NOT_TRACKED || p2.x == XDI_POINT_NOT_TRACKED)
			{
				xdi_position_state = XDI_PSTATE_TRACKING;
			}		
			break;
		}
		case XDI_PSTATE_TRACKING:
		{
			if(p1.x != XDI_POINT_NOT_TRACKED)
			{
				xdi_ui_position_callback(p1.x, p1.y, (p1.x == XDI_POINT_NOT_TRACKED)?1:0);
			}		
			break;
		}
	}
}


typedef enum
{
	XDI_GSTATE_WAIT_FOR_START,
	XDI_GSTATE_WAIT_FOR_END,
} XdiGestrueStates;

XdiGestrueStates xdi_gesture_state = XDI_GSTATE_WAIT_FOR_START;
XdiPoint xdi_gesture_start_p1;
XdiPoint xdi_gesture_start_p2;
XdiPoint xdi_gesture_end_p1;
XdiPoint xdi_gesture_end_p2;

void xdiGestureProcessor(XdiPoint p1, XdiPoint p2)
{
	switch(xdi_gesture_state)
	{
		case XDI_GSTATE_WAIT_FOR_START:
		{
			if(p1.x != XDI_POINT_NOT_TRACKED && p2.x != XDI_POINT_NOT_TRACKED)
			{
				xdi_gesture_start_p1.x = p1.x;
				xdi_gesture_start_p1.y = p1.y;
				xdi_gesture_start_p2.x = p2.x;
				xdi_gesture_start_p2.y = p2.y;

				xdi_gesture_state = XDI_GSTATE_WAIT_FOR_END;
			}
			break;
		}
		case XDI_GSTATE_WAIT_FOR_END:
		{
			if(p1.x == XDI_POINT_NOT_TRACKED || p2.x == XDI_POINT_NOT_TRACKED)
			{
				int between_distance = xdiCalculateDistance(xdi_gesture_end_p1, xdi_gesture_end_p2);
				int between_angle = xdiCalculateAngle(xdi_gesture_end_p1, xdi_gesture_end_p2);

				int movement_distance = xdiCalculateDistance(xdi_gesture_start_p1, xdi_gesture_end_p1);
				int movement_angle = xdiCalculateAngle(xdi_gesture_start_p1, xdi_gesture_end_p1);
				//movement_angle = 360 - movement_angle;
/*
				char data[32];
				sprintf(data, "P1:  %4i %4i", xdi_gesture_end_p1.x, xdi_gesture_end_p1.y);
				glcdWrite(data, 0 , 1);
				sprintf(data, "SP1: %4i %4i", xdi_gesture_start_p1.x, xdi_gesture_start_p1.y);
				glcdWrite(data, 1 , 1);
				sprintf(data, "P2:  %4i %4i", xdi_gesture_end_p2.x, xdi_gesture_end_p2.y);
				glcdWrite(data, 2 , 1);
				sprintf(data, "SP2: %4i %4i", xdi_gesture_start_p2.x, xdi_gesture_start_p2.y);
				glcdWrite(data, 3 , 1);
				sprintf(data, "<--> %4i %3iGrad", between_distance, between_angle);
				glcdWrite(data, 4 , 1);
				sprintf(data, "---> %4i %3iGrad", movement_distance, movement_angle);
				glcdWrite(data, 5 , 1);
*/
				if(movement_distance > 120) //Adjust this value depending on the distance
				{
					unsigned char cmd = 0;

					if(movement_angle >= 335 || movement_angle < 25) //first sector
					{
						cmd = 2; //0
					}
					else if(movement_angle >= 25 && movement_angle < 65) //first sector
					{
						cmd = 6;
					}
					else if(movement_angle >= 65 && movement_angle < 115) //first sector
					{
						cmd = 3;
					}
					else if(movement_angle >= 115 && movement_angle < 155) //first sector
					{
						cmd = 7;
					}
					else if(movement_angle >= 155 && movement_angle < 205) //first sector
					{
						cmd = 0;
					}
					else if(movement_angle >= 205 && movement_angle < 245) //first sector
					{
						cmd = 5;
					}
					else if(movement_angle >= 245 && movement_angle < 295) //first sector
					{
						cmd = 1;
					}
					else if(movement_angle >= 295 && movement_angle < 335) //first sector
					{
						cmd = 4;
					}

					xdi_ui_callback(cmd);					
/*
					sprintf(data, "CMD:%i", cmd);
					glcdWrite(data, 0 , 90);
*/
				}
				else
				{	
					

					
				}				

				xdi_gesture_state = XDI_GSTATE_WAIT_FOR_START;
			}	
			else
			{
				xdi_gesture_end_p1.x = p1.x;
				xdi_gesture_end_p1.y = p1.y;
				xdi_gesture_end_p2.x = p2.x;
				xdi_gesture_end_p2.y = p2.y;
			}
			break;
		}
	}
}

void xdiProcessor(XdiPoint p1, XdiPoint p2)
{
	xdiGestureProcessor(p1, p2);
	xdiPositionProcessor(p1, p2);
}


void xdiMain()
{
	XdiPoint p1, p2;
	unsigned char i;
	unsigned char point = 0;


	if(irsPresent(1))
	{

		// Update sensor values
		irsUpdate();
	
		p1.x = XDI_POINT_NOT_TRACKED;
		p1.y = XDI_POINT_NOT_TRACKED;
		p2.x = XDI_POINT_NOT_TRACKED;
		p2.y = XDI_POINT_NOT_TRACKED;


		// Read values of the channels and put the first to point to p1 and p2
		for(i = 0; i < 4; i++)
		{
			if(irs_points[i].xpos != 1023 && irs_points[i].ypos != 1023)
			{
			/*
				unsigned char data[5];
				data[0] = irs_points[i].xpos >> 8;
				data[1] = irs_points[i].xpos & 0xFF;
				data[2] = irs_points[i].ypos >> 8;
				data[3] = irs_points[i].ypos & 0xFF;
				data[4] = i;
				sieSubmit(0, 0x53, 5, data);
			*/
				if(point == 0)
				{
					p1.x = irs_points[i].xpos;
					p1.y = irs_points[i].ypos;

					point++;
				}
				else
				{
					p2.x = irs_points[i].xpos;
					p2.y = irs_points[i].ypos;

					point++;
					break;				
				}
			}
		}
	
		

		xdiProcessor(p1, p2);
	}
}






int xdiCalculateDistance(XdiPoint p1, XdiPoint p2)
{
	int delta_x = p1.x - p2.x; //Use long here --> easier for calculation
	int delta_y = p1.y - p2.y; //Use long here --> easier for calculation

	delta_x = (delta_x < 0)?(-delta_x):delta_x; //Absolute value
	delta_y = (delta_y < 0)?(-delta_y):delta_y; //Absolute value

	return (int)(sqrt((double)(((unsigned long)delta_x*(unsigned long)delta_x) + ((unsigned long)delta_y*(unsigned long)delta_y))));

/*
	unsigned long delta_x_pow;
	unsigned long delta_y_pow;

	double sum;
	double sqr;

	int ret;

	delta_x_pow = (unsigned long)delta_x*(unsigned long)delta_x;
	delta_y_pow = (unsigned long)delta_y*(unsigned long)delta_y;

	sum = (double)(delta_x_pow + delta_y_pow);

	sqr = sqrt(sum);

	ret = (int)sqr;

	return ret;
*/
}

int xdiCalculateAngle(XdiPoint p1, XdiPoint p2)
{
	int delta_x = p1.x - p2.x;
	int delta_y = p1.y - p2.y;


	return atan2((double)delta_y,(double)delta_x)/M_PI*180+180;
}

